@@ -3,6 +3,7 @@ from collections import namedtuple
 from dataclasses import dataclass
 from typing import Any, Dict, List, Optional, Set, Tuple, Union, cast
 
+from samtranslator.feature_toggle.feature_toggle import FeatureToggle
 from samtranslator.metrics.method_decorator import cw_timer
 from samtranslator.model import Resource
 from samtranslator.model.apigateway import (
@@ -40,6 +41,8 @@ from samtranslator.validator.value_validator import sam_expect
 
 LOG = logging.getLogger(__name__)
 
+FEATURE_FLAG_NORMALIZED_OPENAPI_VERSION = "normalized_open_api_version"
+
 _CORS_WILDCARD = "'*'"
 CorsProperties = namedtuple(
     "CorsProperties", ["AllowMethods", "AllowHeaders", "AllowOrigin", "MaxAge", "AllowCredentials"]
@@ -169,7 +172,7 @@ class SharedApiUsagePlan:
 
 
 class ApiGenerator:
-    def __init__(  # noqa: too-many-arguments
+    def __init__(  # noqa: PLR0913
         self,
         logical_id: str,
         cache_cluster_enabled: Optional[Intrinsicable[bool]],
@@ -205,6 +208,7 @@ class ApiGenerator:
         mode: Optional[Intrinsicable[str]] = None,
         api_key_source_type: Optional[Intrinsicable[str]] = None,
         always_deploy: Optional[bool] = False,
+        feature_toggle: Optional[FeatureToggle] = None,
     ):
         """Constructs an API Generator class that generates API Gateway resources
 
@@ -261,6 +265,7 @@ class ApiGenerator:
         self.mode = mode
         self.api_key_source_type = api_key_source_type
         self.always_deploy = always_deploy
+        self.feature_toggle = feature_toggle
 
     def _construct_rest_api(self) -> ApiGatewayRestApi:
         """Constructs and returns the ApiGateway RestApi.
@@ -450,7 +455,7 @@ class ApiGenerator:
 
         return stage
 
-    def _construct_api_domain(  # noqa: too-many-branches
+    def _construct_api_domain(  # noqa: PLR0912, PLR0915
         self, rest_api: ApiGatewayRestApi, route53_record_set_groups: Any
     ) -> ApiDomainResponse:
         """
@@ -604,9 +609,7 @@ class ApiGenerator:
         hostedZoneId = route53.get("HostedZoneId")
         hostedZoneName = route53.get("HostedZoneName")
         domainName = domain.get("DomainName")
-        logical_id = logical_id = LogicalIdGenerator(
-            "RecordSetGroup", [hostedZoneId or hostedZoneName, domainName]
-        ).gen()
+        logical_id = LogicalIdGenerator("RecordSetGroup", [hostedZoneId or hostedZoneName, domainName]).gen()
 
         record_set_group = Route53RecordSetGroup(logical_id, attributes=self.passthrough_resource_attributes)
         if hostedZoneId:
@@ -747,8 +750,9 @@ class ApiGenerator:
             properties = CorsProperties(AllowOrigin=self.cors)  # type: ignore[call-arg]
         elif isinstance(self.cors, dict):
             # Make sure keys in the dict are recognized
-            if not all(key in CorsProperties._fields for key in self.cors):
-                raise InvalidResourceException(self.logical_id, INVALID_ERROR)
+            for key in self.cors:
+                if key not in CorsProperties._fields:
+                    raise InvalidResourceException(self.logical_id, f"Invalid key '{key}' for 'Cors' property.")
 
             properties = CorsProperties(**self.cors)
 
@@ -856,7 +860,7 @@ class ApiGenerator:
 
         self.definition_body = self._openapi_postprocess(swagger_editor.swagger)
 
-    def _construct_usage_plan(self, rest_api_stage: Optional[ApiGatewayStage] = None) -> Any:  # noqa: too-many-branches
+    def _construct_usage_plan(self, rest_api_stage: Optional[ApiGatewayStage] = None) -> Any:  # noqa: PLR0912
         """Constructs and returns the ApiGateway UsagePlan, ApiGateway UsagePlanKey, ApiGateway ApiKey for Auth.
 
         :param model.apigateway.ApiGatewayStage stage: the stage of rest api
@@ -1044,16 +1048,14 @@ class ApiGenerator:
             if not isinstance(responses_value, dict):
                 raise InvalidResourceException(
                     self.logical_id,
-                    "Invalid property type '{}' for GatewayResponses. "
-                    "Expected an object of type 'GatewayResponse'.".format(type(responses_value).__name__),
+                    f"Invalid property type '{type(responses_value).__name__}' for GatewayResponses. "
+                    "Expected an object of type 'GatewayResponse'.",
                 )
             for response_key in responses_value:
                 if response_key not in GatewayResponseProperties:
                     raise InvalidResourceException(
                         self.logical_id,
-                        "Invalid property '{}' in 'GatewayResponses' property '{}'.".format(
-                            response_key, responses_key
-                        ),
+                        f"Invalid property '{response_key}' in 'GatewayResponses' property '{responses_key}'.",
                     )
 
         if not SwaggerEditor.is_valid(self.definition_body):
@@ -1119,7 +1121,7 @@ class ApiGenerator:
 
         self.definition_body = self._openapi_postprocess(swagger_editor.swagger)
 
-    def _openapi_postprocess(self, definition_body: Dict[str, Any]) -> Dict[str, Any]:  # noqa: too-many-branches
+    def _openapi_postprocess(self, definition_body: Dict[str, Any]) -> Dict[str, Any]:  # noqa: PLR0912
         """
         Convert definitions to openapi 3 in definition body if OpenApiVersion flag is specified.
 
@@ -1129,11 +1131,15 @@ class ApiGenerator:
         if definition_body.get("swagger") is not None:
             return definition_body
 
-        if definition_body.get("openapi") is not None and self.open_api_version is None:
-            self.open_api_version = definition_body.get("openapi")
+        if self.feature_toggle and self.feature_toggle.is_enabled(FEATURE_FLAG_NORMALIZED_OPENAPI_VERSION):
+            normalized_open_api_version = definition_body.get("openapi", self.open_api_version)
+        elif definition_body.get("openapi") is not None and self.open_api_version is None:
+            normalized_open_api_version = definition_body.get("openapi")
+        else:
+            normalized_open_api_version = self.open_api_version
 
-        if self.open_api_version and SwaggerEditor.safe_compare_regex_with_string(
-            SwaggerEditor._OPENAPI_VERSION_3_REGEX, self.open_api_version
+        if normalized_open_api_version and SwaggerEditor.safe_compare_regex_with_string(
+            SwaggerEditor._OPENAPI_VERSION_3_REGEX, normalized_open_api_version
         ):
             if definition_body.get("securityDefinitions"):
                 components = definition_body.get("components", Py27Dict())
@@ -1168,8 +1174,8 @@ class ApiGenerator:
                     if path_item.get("options"):
                         SwaggerEditor.validate_is_dict(
                             path_item.get("options"),
-                            "Value of options method for path {} must be a "
-                            "dictionary according to Swagger spec.".format(path),
+                            f"Value of options method for path {path} must be a "
+                            "dictionary according to Swagger spec.",
                         )
                         options = path_item.get("options").copy()
                         for field, field_val in options.items():
@@ -1184,7 +1190,7 @@ class ApiGenerator:
                                     raise InvalidDocumentException(
                                         [
                                             InvalidTemplateException(
-                                                f"Invalid responses in options method for path {path}: {str(ex)}.",
+                                                f"Invalid responses in options method for path {path}: {ex!s}.",
                                             )
                                         ]
                                     ) from ex
@@ -1192,8 +1198,8 @@ class ApiGenerator:
                                     continue
                                 SwaggerEditor.validate_is_dict(
                                     response_200_headers,
-                                    "Value of response's headers in options method for path {} must be a "
-                                    "dictionary according to Swagger spec.".format(path),
+                                    f"Value of response's headers in options method for path {path} must be a "
+                                    "dictionary according to Swagger spec.",
                                 )
                                 for header, header_val in response_200_headers.items():
                                     new_header_val_with_schema = Py27Dict()
@@ -1328,8 +1334,19 @@ class ApiGenerator:
         if isinstance(value, dict) and value.get("Type"):
             rest_api.Parameters = {"endpointConfigurationTypes": value.get("Type")}
             rest_api.EndpointConfiguration = {"Types": [value.get("Type")]}
-            if "VPCEndpointIds" in value:
-                rest_api.EndpointConfiguration["VpcEndpointIds"] = value.get("VPCEndpointIds")
+
+            # SAM API `EndpointConfiguration` uses `VPCEndpointIds` but APIGateway RestApi uses `VpcEndpointIds`.
+            # Deny when both properties are defined at the same time.
+            if "VPCEndpointIds" in value and "VpcEndpointIds" in value:
+                raise InvalidResourceException(
+                    rest_api.logical_id,
+                    "'VPCEndpointIds' and 'VpcEndpointIds' cannot be used together in EndpointConfiguration.",
+                )
+            # Accept when either `VPCEndpointIds` or `VpcEndpointIds` is defined by users
+            if "VPCEndpointIds" in value or "VpcEndpointIds" in value:
+                rest_api.EndpointConfiguration["VpcEndpointIds"] = value.get("VPCEndpointIds") or value.get(
+                    "VpcEndpointIds"
+                )
         else:
             rest_api.EndpointConfiguration = {"Types": [value]}
             rest_api.Parameters = {"endpointConfigurationTypes": value}
