@@ -28,6 +28,7 @@ class ApiGatewayRestApi(Resource):
         "MinimumCompressionSize": GeneratedProperty(),
         "Mode": GeneratedProperty(),
         "ApiKeySourceType": GeneratedProperty(),
+        "Tags": GeneratedProperty(),
     }
 
     Body: Optional[Dict[str, Any]]
@@ -42,6 +43,7 @@ class ApiGatewayRestApi(Resource):
     MinimumCompressionSize: Optional[PassThrough]
     Mode: Optional[PassThrough]
     ApiKeySourceType: Optional[PassThrough]
+    Tags: Optional[PassThrough]
 
     runtime_attrs = {"rest_api_id": lambda self: ref(self.logical_id)}
 
@@ -90,7 +92,7 @@ class ApiGatewayDeployment(Resource):
 
     runtime_attrs = {"deployment_id": lambda self: ref(self.logical_id)}
 
-    def make_auto_deployable(  # noqa: too-many-arguments
+    def make_auto_deployable(  # noqa: PLR0913
         self,
         stage: ApiGatewayStage,
         openapi_version: Optional[Union[Dict[str, Any], str]] = None,
@@ -214,6 +216,7 @@ class ApiGatewayDomainName(Resource):
         "MutualTlsAuthentication": GeneratedProperty(),
         "SecurityPolicy": GeneratedProperty(),
         "CertificateArn": GeneratedProperty(),
+        "Tags": GeneratedProperty(),
         "OwnershipVerificationCertificateArn": GeneratedProperty(),
     }
 
@@ -223,6 +226,7 @@ class ApiGatewayDomainName(Resource):
     MutualTlsAuthentication: Optional[Dict[str, Any]]
     SecurityPolicy: Optional[PassThrough]
     CertificateArn: Optional[PassThrough]
+    Tags: Optional[PassThrough]
     OwnershipVerificationCertificateArn: Optional[PassThrough]
 
 
@@ -266,6 +270,7 @@ class ApiGatewayApiKey(Resource):
         "Enabled": GeneratedProperty(),
         "GenerateDistinctId": GeneratedProperty(),
         "Name": GeneratedProperty(),
+        "Tags": GeneratedProperty(),
         "StageKeys": GeneratedProperty(),
         "Value": GeneratedProperty(),
     }
@@ -276,7 +281,7 @@ class ApiGatewayApiKey(Resource):
 class ApiGatewayAuthorizer:
     _VALID_FUNCTION_PAYLOAD_TYPES = [None, "TOKEN", "REQUEST"]
 
-    def __init__(  # type: ignore[no-untyped-def]# noqa: too-many-arguments
+    def __init__(  # type: ignore[no-untyped-def]# noqa: PLR0913
         self,
         api_logical_id=None,
         name=None,
